@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012, 2013 Red Hat, Inc.
+ * Copyright © 2012, 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,12 +37,18 @@
 #include "photos-base-item.h"
 #include "photos-camera-cache.h"
 #include "photos-dlna-renderers-dialog.h"
+#include "photos-filterable.h"
 #include "photos-gom-miner.h"
 #include "photos-item-manager.h"
 #include "photos-main-window.h"
 #include "photos-mode-controller.h"
 #include "photos-properties-dialog.h"
+#include "photos-query.h"
 #include "photos-resources.h"
+#include "photos-search-context.h"
+#include "photos-search-controller.h"
+#include "photos-search-provider.h"
+#include "photos-single-item-job.h"
 #include "photos-source-manager.h"
 
 
@@ -65,13 +71,20 @@ struct _PhotosApplicationPrivate
   GomMiner *flickr_miner;
   GtkWidget *main_window;
   PhotosBaseManager *item_mngr;
-  PhotosBaseManager *src_mngr;
   PhotosCameraCache *camera_cache;
   PhotosModeController *mode_cntrlr;
+  PhotosSearchContextState *state;
+  PhotosSearchProvider *search_provider;
+  guint32 activation_timestamp;
 };
 
+static void photos_application_search_context_iface_init (PhotosSearchContextInterface *iface);
 
-G_DEFINE_TYPE_WITH_PRIVATE (PhotosApplication, photos_application, GTK_TYPE_APPLICATION)
+
+G_DEFINE_TYPE_WITH_CODE (PhotosApplication, photos_application, GTK_TYPE_APPLICATION,
+                         G_ADD_PRIVATE (PhotosApplication)
+                         G_IMPLEMENT_INTERFACE (PHOTOS_TYPE_SEARCH_CONTEXT,
+                                                photos_application_search_context_iface_init));
 
 
 enum
@@ -88,6 +101,8 @@ struct _PhotosApplicationRefreshData
 };
 
 static gboolean photos_application_refresh_miner_now (PhotosApplication *self, GomMiner *miner);
+static void photos_application_start_miners (PhotosApplication *self);
+static void photos_application_stop_miners (PhotosApplication *self);
 
 
 static PhotosApplicationRefreshData *
@@ -134,6 +149,103 @@ photos_application_action_toggle (GSimpleAction *simple, GVariant *parameter, gp
 }
 
 
+static void
+photos_application_destroy (PhotosApplication *self)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+
+  priv->main_window = NULL;
+
+  if (priv->miners_running != NULL)
+    {
+      photos_application_stop_miners (self);
+      g_list_free_full (priv->miners_running, g_object_unref);
+      priv->miners_running = NULL;
+    }
+}
+
+
+static void
+photos_application_create_window (PhotosApplication *self)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+
+  if (priv->main_window != NULL)
+    return;
+
+  priv->main_window = photos_main_window_new (GTK_APPLICATION (self));
+  g_signal_connect_swapped (priv->main_window, "destroy", G_CALLBACK (photos_application_destroy), self);
+  photos_application_start_miners (self);
+}
+
+
+static void
+photos_application_activate_item (PhotosApplication *self, GObject *item)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+
+  photos_base_manager_set_active_object (priv->item_mngr, item);
+
+  photos_application_create_window (self);
+  photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_PREVIEW);
+  g_application_activate (G_APPLICATION (self));
+
+  /* TODO: Forward the search terms when we exit the preview */
+}
+
+
+static void
+photos_application_activate_query_executed (TrackerSparqlCursor *cursor, gpointer user_data)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+  PhotosApplicationPrivate *priv = self->priv;
+  GObject *item;
+  const gchar *identifier;
+
+  if (cursor == NULL)
+    goto out;
+
+  photos_item_manager_add_item (PHOTOS_ITEM_MANAGER (priv->item_mngr), cursor);
+
+  identifier = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_URN, NULL);
+  item = photos_base_manager_get_object_by_id (priv->item_mngr, identifier);
+
+  photos_application_activate_item (self, item);
+
+ out:
+  g_object_unref (self);
+}
+
+
+static void
+photos_application_activate_result (PhotosApplication *self,
+                                    const gchar *identifier,
+                                    const gchar *const *terms,
+                                    guint timestamp)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+  GObject *item;
+
+  priv->activation_timestamp = timestamp;
+
+  item = photos_base_manager_get_object_by_id (priv->item_mngr, identifier);
+  if (item != NULL)
+    photos_application_activate_item (self, item);
+  else
+    {
+      PhotosSingleItemJob *job;
+
+      job = photos_single_item_job_new (identifier);
+      photos_single_item_job_run (job,
+                                  priv->state,
+                                  PHOTOS_QUERY_FLAGS_UNFILTERED,
+                                  photos_application_activate_query_executed,
+                                  g_object_ref (self));
+      g_object_unref (job);
+    }
+}
+
+
 static void
 photos_application_can_fullscreen_changed (PhotosApplication *self)
 {
@@ -154,6 +266,51 @@ photos_application_fullscreen (PhotosApplication *self, GVariant *parameter)
 }
 
 
+static PhotosSearchContextState *
+photos_application_get_state (PhotosSearchContext *context)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (context);
+  return self->priv->state;
+}
+
+
+static void
+photos_application_init_app_menu (PhotosApplication *self)
+{
+  GMenu *menu;
+  GtkBuilder *builder;
+
+  builder = gtk_builder_new ();
+  gtk_builder_add_from_resource (builder, "/org/gnome/photos/app-menu.ui", NULL);
+
+  menu = G_MENU (gtk_builder_get_object (builder, "app-menu"));
+  gtk_application_set_app_menu (GTK_APPLICATION (self), G_MENU_MODEL (menu));
+  g_object_unref (builder);
+}
+
+
+static void
+photos_application_launch_search (PhotosApplication *self, const gchar* const *terms, guint timestamp)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+  GVariant *state;
+  gchar *str;
+
+  photos_application_create_window (self);
+  photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
+
+  str = g_strjoinv (" ", (gchar **) terms);
+  photos_search_controller_set_string (priv->state->srch_cntrlr, str);
+  g_free (str);
+
+  state = g_variant_new ("b", TRUE);
+  g_action_group_change_action_state (G_ACTION_GROUP (self), "search", state);
+
+  priv->activation_timestamp = timestamp;
+  g_application_activate (G_APPLICATION (self));
+}
+
+
 static void
 photos_application_open_current (PhotosApplication *self)
 {
@@ -190,15 +347,15 @@ static void
 photos_application_properties (PhotosApplication *self)
 {
   PhotosApplicationPrivate *priv = self->priv;
+  GObject *item;
   GtkWidget *dialog;
-  PhotosBaseItem *item;
   const gchar *id;
 
-  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->item_mngr));
+  item = photos_base_manager_get_active_object (priv->item_mngr);
   if (item == NULL)
     return;
 
-  id = photos_base_item_get_id (item);
+  id = photos_filterable_get_id (PHOTOS_FILTERABLE (item));
   dialog = photos_properties_dialog_new (GTK_WINDOW (priv->main_window), id);
   gtk_widget_show_all (dialog);
   g_signal_connect (dialog, "response", G_CALLBACK (gtk_widget_destroy), NULL);
@@ -271,10 +428,10 @@ photos_application_refresh_miners (PhotosApplication *self)
 {
   PhotosApplicationPrivate *priv = self->priv;
 
-  if (photos_source_manager_has_provider_type (PHOTOS_SOURCE_MANAGER (priv->src_mngr), "facebook"))
+  if (photos_source_manager_has_provider_type (PHOTOS_SOURCE_MANAGER (priv->state->src_mngr), "facebook"))
     photos_application_refresh_miner_now (self, priv->facebook_miner);
 
-  if (photos_source_manager_has_provider_type (PHOTOS_SOURCE_MANAGER (priv->src_mngr), "flickr"))
+  if (photos_source_manager_has_provider_type (PHOTOS_SOURCE_MANAGER (priv->state->src_mngr), "flickr"))
     photos_application_refresh_miner_now (self, priv->flickr_miner);
 }
 
@@ -283,15 +440,15 @@ static void
 photos_application_remote_display_current (PhotosApplication *self)
 {
   PhotosApplicationPrivate *priv = self->priv;
-  PhotosBaseItem *item;
+  GObject *item;
   GtkWidget *dialog;
   const gchar *urn;
 
-  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->item_mngr));
+  item = photos_base_manager_get_active_object (priv->item_mngr);
   if (item == NULL)
     return;
 
-  urn = photos_base_item_get_id (item);
+  urn = photos_filterable_get_id (PHOTOS_FILTERABLE (item));
   dialog = photos_dlna_renderers_dialog_new (GTK_WINDOW (priv->main_window), urn);
   gtk_widget_show_all (dialog);
 }
@@ -358,12 +515,12 @@ photos_application_start_miners (PhotosApplication *self)
 
   photos_application_refresh_miners (self);
 
-  g_signal_connect_object (priv->src_mngr,
+  g_signal_connect_object (priv->state->src_mngr,
                            "object-added",
                            G_CALLBACK (photos_application_refresh_miners),
                            self,
                            G_CONNECT_SWAPPED);
-  g_signal_connect_object (priv->src_mngr,
+  g_signal_connect_object (priv->state->src_mngr,
                            "object-removed",
                            G_CALLBACK (photos_application_refresh_miners),
                            self,
@@ -415,7 +572,59 @@ photos_application_activate (GApplication *application)
   PhotosApplication *self = PHOTOS_APPLICATION (application);
   PhotosApplicationPrivate *priv = self->priv;
 
-  gtk_window_present (GTK_WINDOW (priv->main_window));
+  if (priv->main_window == NULL)
+    {
+      photos_application_create_window (self);
+      photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
+    }
+
+  gtk_window_present_with_time (GTK_WINDOW (priv->main_window), priv->activation_timestamp);
+  priv->activation_timestamp = GDK_CURRENT_TIME;
+}
+
+
+static gboolean
+photos_application_dbus_register (GApplication *application,
+                                  GDBusConnection *connection,
+                                  const gchar *object_path,
+                                  GError **error)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (application);
+  gboolean ret_val = FALSE;
+  gchar *search_provider_path = NULL;
+
+  if (!G_APPLICATION_CLASS (photos_application_parent_class)->dbus_register (application,
+                                                                             connection,
+                                                                             object_path,
+                                                                             error))
+    goto out;
+
+  search_provider_path = g_strconcat (object_path, PHOTOS_SEARCH_PROVIDER_PATH_SUFFIX, NULL);
+  if (!photos_search_provider_dbus_export (self->priv->search_provider, connection, search_provider_path, error))
+    goto out;
+
+  ret_val = TRUE;
+
+ out:
+  g_free (search_provider_path);
+  return ret_val;
+}
+
+
+static void
+photos_application_dbus_unregister (GApplication *application,
+                                    GDBusConnection *connection,
+                                    const gchar *object_path)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (application);
+  gchar *search_provider_path = NULL;
+
+  search_provider_path = g_strconcat (object_path, PHOTOS_SEARCH_PROVIDER_PATH_SUFFIX, NULL);
+  photos_search_provider_dbus_unexport (self->priv->search_provider, connection, search_provider_path);
+
+  G_APPLICATION_CLASS (photos_application_parent_class)->dbus_unregister (application, connection, object_path);
+
+  g_free (search_provider_path);
 }
 
 
@@ -425,10 +634,8 @@ photos_application_startup (GApplication *application)
   PhotosApplication *self = PHOTOS_APPLICATION (application);
   PhotosApplicationPrivate *priv = self->priv;
   GError *error;
-  GMenu *menu;
   GSimpleAction *action;
   GrlRegistry *registry;
-  GtkBuilder *builder;
   GtkSettings *settings;
   GVariant *state;
 
@@ -469,7 +676,6 @@ photos_application_startup (GApplication *application)
                                                          NULL);
 
   priv->item_mngr = photos_item_manager_dup_singleton ();
-  priv->src_mngr = photos_source_manager_dup_singleton ();
 
   /* A dummy reference to keep it alive during the lifetime of the
    * application.
@@ -538,12 +744,7 @@ photos_application_startup (GApplication *application)
                             G_CALLBACK (photos_application_window_mode_changed),
                             self);
 
-  builder = gtk_builder_new ();
-  gtk_builder_add_from_resource (builder, "/org/gnome/photos/app-menu.ui", NULL);
-
-  menu = G_MENU (gtk_builder_get_object (builder, "app-menu"));
-  gtk_application_set_app_menu (GTK_APPLICATION (self), G_MENU_MODEL (menu));
-  g_object_unref (builder);
+  photos_application_init_app_menu (self);
 
   gtk_application_add_accelerator (GTK_APPLICATION (self), "<Primary>q", "app.quit", NULL);
   gtk_application_add_accelerator (GTK_APPLICATION (self), "F11", "app.fullscreen", NULL);
@@ -551,11 +752,6 @@ photos_application_startup (GApplication *application)
   gtk_application_add_accelerator (GTK_APPLICATION (self), "<Primary>p", "app.print-current", NULL);
   gtk_application_add_accelerator (GTK_APPLICATION (self), "<Primary>f", "app.search", NULL);
   gtk_application_add_accelerator (GTK_APPLICATION (self), "<Primary>a", "app.select-all", NULL);
-
-  priv->main_window = photos_main_window_new (GTK_APPLICATION (self));
-  photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_OVERVIEW);
-
-  photos_application_start_miners (self);
 }
 
 
@@ -583,13 +779,6 @@ photos_application_dispose (GObject *object)
   PhotosApplication *self = PHOTOS_APPLICATION (object);
   PhotosApplicationPrivate *priv = self->priv;
 
-  if (priv->miners_running != NULL)
-    {
-      photos_application_stop_miners (self);
-      g_list_free_full (priv->miners_running, g_object_unref);
-      priv->miners_running = NULL;
-    }
-
   if (priv->resource != NULL)
     {
       g_resources_unregister (priv->resource);
@@ -610,9 +799,15 @@ photos_application_dispose (GObject *object)
   g_clear_object (&priv->facebook_miner);
   g_clear_object (&priv->flickr_miner);
   g_clear_object (&priv->item_mngr);
-  g_clear_object (&priv->src_mngr);
   g_clear_object (&priv->camera_cache);
   g_clear_object (&priv->mode_cntrlr);
+  g_clear_object (&priv->search_provider);
+
+  if (priv->state != NULL)
+    {
+      photos_search_context_state_free (priv->state);
+      priv->state = NULL;
+    }
 
   G_OBJECT_CLASS (photos_application_parent_class)->dispose (object);
 }
@@ -621,8 +816,25 @@ photos_application_dispose (GObject *object)
 static void
 photos_application_init (PhotosApplication *self)
 {
+  PhotosApplicationPrivate *priv;
+
   self->priv = photos_application_get_instance_private (self);
+  priv = self->priv;
+
   eog_debug_init ();
+
+  priv->search_provider = photos_search_provider_new ();
+  g_signal_connect_swapped (priv->search_provider,
+                            "activate-result",
+                            G_CALLBACK (photos_application_activate_result),
+                            self);
+  g_signal_connect_swapped (priv->search_provider,
+                            "launch-search",
+                            G_CALLBACK (photos_application_launch_search),
+                            self);
+
+  priv->state = photos_search_context_state_new (PHOTOS_SEARCH_CONTEXT (self));
+  priv->activation_timestamp = GDK_CURRENT_TIME;
 }
 
 
@@ -635,17 +847,26 @@ photos_application_class_init (PhotosApplicationClass *class)
   object_class->constructor = photos_application_constructor;
   object_class->dispose = photos_application_dispose;
   application_class->activate = photos_application_activate;
+  application_class->dbus_register = photos_application_dbus_register;
+  application_class->dbus_unregister = photos_application_dbus_unregister;
   application_class->startup = photos_application_startup;
 
   /* TODO: Add miners-changed signal */
 }
 
 
+static void
+photos_application_search_context_iface_init (PhotosSearchContextInterface *iface)
+{
+  iface->get_state = photos_application_get_state;
+}
+
+
 GtkApplication *
 photos_application_new (void)
 {
   return g_object_new (PHOTOS_TYPE_APPLICATION,
                        "application-id", "org.gnome." PACKAGE_NAME,
-                       "flags", G_APPLICATION_FLAGS_NONE,
+                       "flags", G_APPLICATION_IS_SERVICE,
                        NULL);
 }
