@@ -30,14 +30,17 @@
 #include <gio/gio.h>
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <libgd/gd.h>
 #include <tracker-sparql.h>
 
 #include "photos-base-item.h"
 #include "photos-collection-icon-watcher.h"
 #include "photos-delete-item-job.h"
+#include "photos-filterable.h"
 #include "photos-icons.h"
 #include "photos-print-operation.h"
 #include "photos-query.h"
+#include "photos-search-context.h"
 #include "photos-selection-controller.h"
 #include "photos-single-item-job.h"
 #include "photos-utils.h"
@@ -60,7 +63,6 @@ struct _PhotosBaseItemPrivate
   gboolean collection;
   gboolean failed_thumbnailing;
   gboolean favorite;
-  gboolean thumbnailed;
   const gchar *thumb_path;
   gchar *author;
   gchar *default_app_name;
@@ -98,8 +100,12 @@ enum
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+static void photos_base_item_filterable_iface_init (PhotosFilterableInterface *iface);
 
-G_DEFINE_TYPE_WITH_PRIVATE (PhotosBaseItem, photos_base_item, G_TYPE_OBJECT);
+
+G_DEFINE_TYPE_WITH_CODE (PhotosBaseItem, photos_base_item, G_TYPE_OBJECT,
+                         G_ADD_PRIVATE (PhotosBaseItem)
+                         G_IMPLEMENT_INTERFACE (PHOTOS_TYPE_FILTERABLE, photos_base_item_filterable_iface_init));
 
 
 static GThreadPool *create_thumbnail_pool;
@@ -193,15 +199,15 @@ photos_base_item_check_effects_and_update_info (PhotosBaseItem *self)
 
   g_object_unref (priv->icon);
 
-  if (priv->thumbnailed)
+  if (priv->thumb_path != NULL)
     {
       GtkBorder *slice;
 
       slice = photos_utils_get_thumbnail_frame_border ();
-      priv->icon = photos_utils_embed_image_in_frame (icon,
-                                                      PACKAGE_ICONS_DIR "/thumbnail-frame.png",
-                                                      slice,
-                                                      slice);
+      priv->icon = gd_embed_image_in_frame (icon,
+                                            "resource:///org/gnome/photos/thumbnail-frame.png",
+                                            slice,
+                                            slice);
       gtk_border_free (slice);
     }
   else
@@ -337,6 +343,14 @@ photos_base_item_download_in_thread_func (GTask *task,
 }
 
 
+static const gchar *
+photos_base_item_get_id (PhotosFilterable *filterable)
+{
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (filterable);
+  return self->priv->id;
+}
+
+
 static void
 photos_base_item_icon_updated (PhotosBaseItem *self, GIcon *icon)
 {
@@ -392,12 +406,20 @@ photos_base_item_refresh_thumb_path_pixbuf (GObject *source_object, GAsyncResult
   priv->icon = gdk_pixbuf_new_from_stream_finish (res, &error);
   if (error != NULL)
     {
+      GFile *file;
+      gchar *uri;
+
+      file = G_FILE (g_object_get_data (G_OBJECT (stream), "file"));
+      uri = g_file_get_uri (file);
+      g_warning ("Unable to create pixbuf from %s: %s", uri, error->message);
       priv->failed_thumbnailing = TRUE;
+      priv->thumb_path = NULL;
+      g_file_delete_async (file, G_PRIORITY_DEFAULT, NULL, NULL, NULL);
+      g_free (uri);
       g_error_free (error);
       goto out;
     }
 
-  priv->thumbnailed = TRUE;
   photos_base_item_check_effects_and_update_info (self);
 
  out:
@@ -419,11 +441,19 @@ photos_base_item_refresh_thumb_path_read (GObject *source_object, GAsyncResult *
   stream = g_file_read_finish (file, res, &error);
   if (error != NULL)
     {
+      gchar *uri;
+
+      uri = g_file_get_uri (file);
+      g_warning ("Unable to read file at %s: %s", uri, error->message);
       priv->failed_thumbnailing = TRUE;
+      priv->thumb_path = NULL;
+      g_file_delete_async (file, G_PRIORITY_DEFAULT, NULL, NULL, NULL);
+      g_free (uri);
       g_error_free (error);
       goto out;
     }
 
+  g_object_set_data_full (G_OBJECT (stream), "file", g_object_ref (file), g_object_unref);
   size = photos_utils_get_icon_size ();
   gdk_pixbuf_new_from_stream_at_scale_async (G_INPUT_STREAM (stream),
                                              size,
@@ -538,7 +568,6 @@ photos_base_item_file_query_info (GObject *source_object, GAsyncResult *res, gpo
     photos_base_item_refresh_thumb_path (self);
   else
     {
-      priv->thumbnailed = FALSE;
       photos_base_item_create_thumbnail_async (self,
                                                NULL,
                                                photos_base_item_create_thumbnail_cb,
@@ -957,6 +986,13 @@ photos_base_item_class_init (PhotosBaseItemClass *class)
 }
 
 
+static void
+photos_base_item_filterable_iface_init (PhotosFilterableInterface *iface)
+{
+  iface->get_id = photos_base_item_get_id;
+}
+
+
 gboolean
 photos_base_item_can_trash (PhotosBaseItem *self)
 {
@@ -1086,13 +1122,6 @@ photos_base_item_get_icon (PhotosBaseItem *self)
 }
 
 
-const gchar *
-photos_base_item_get_id (PhotosBaseItem *self)
-{
-  return self->priv->id;
-}
-
-
 const gchar *
 photos_base_item_get_identifier (PhotosBaseItem *self)
 {
@@ -1260,10 +1289,19 @@ photos_base_item_print (PhotosBaseItem *self, GtkWidget *toplevel)
 void
 photos_base_item_refresh (PhotosBaseItem *self)
 {
+  GApplication *app;
+  PhotosSearchContextState *state;
   PhotosSingleItemJob *job;
 
+  app = g_application_get_default ();
+  state = photos_search_context_get_state (PHOTOS_SEARCH_CONTEXT (app));
+
   job = photos_single_item_job_new (self->priv->id);
-  photos_single_item_job_run (job, PHOTOS_QUERY_FLAGS_NONE, photos_base_item_refresh_executed, g_object_ref (self));
+  photos_single_item_job_run (job,
+                              state,
+                              PHOTOS_QUERY_FLAGS_NONE,
+                              photos_base_item_refresh_executed,
+                              g_object_ref (self));
   g_object_unref (job);
 }
 
