@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2013 Red Hat, Inc.
+ * Copyright © 2013, 2014 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,12 +28,13 @@
 #include <glib.h>
 #include <tracker-sparql.h>
 
-#include "photos-collection-manager.h"
 #include "photos-fetch-collection-state-job.h"
 #include "photos-fetch-collections-job.h"
+#include "photos-filterable.h"
 #include "photos-item-manager.h"
 #include "photos-query.h"
 #include "photos-query-builder.h"
+#include "photos-search-context.h"
 #include "photos-selection-controller.h"
 
 
@@ -117,7 +118,8 @@ photos_fetch_collection_state_job_emit_callback (PhotosFetchCollectionStateJob *
         {
           item_idx = (gchar *) keys->data;
           item = PHOTOS_BASE_ITEM (photos_base_manager_get_object_by_id (priv->item_mngr, item_idx));
-          if (g_strcmp0 (photos_base_item_get_id (item), photos_base_item_get_id (collection)) == 0)
+          if (g_strcmp0 (photos_filterable_get_id (PHOTOS_FILTERABLE (item)),
+                         photos_filterable_get_id (PHOTOS_FILTERABLE (collection))) == 0)
             hidden = TRUE;
         }
       g_list_free (keys);
@@ -217,16 +219,21 @@ static void
 photos_fetch_collection_state_job_init (PhotosFetchCollectionStateJob *self)
 {
   PhotosFetchCollectionStateJobPrivate *priv = self->priv;
+  GApplication *app;
+  PhotosSearchContextState *state;
 
   self->priv = photos_fetch_collection_state_job_get_instance_private (self);
   priv = self->priv;
 
+  app = g_application_get_default ();
+  state = photos_search_context_get_state (PHOTOS_SEARCH_CONTEXT (app));
+
   priv->collections_for_items = g_hash_table_new_full (g_str_hash,
                                                        g_str_equal,
                                                        g_free,
                                                        photos_fetch_collection_state_job_value_destroy_func);
 
-  priv->col_mngr = photos_collection_manager_dup_singleton ();
+  priv->col_mngr = g_object_ref (state->col_mngr);
   priv->item_mngr = photos_item_manager_dup_singleton ();
   priv->sel_cntrlr = photos_selection_controller_dup_singleton ();
 }
