@@ -30,7 +30,6 @@
 #include <glib/gi18n.h>
 #include <libgd/gd.h>
 
-#include "photos-application.h"
 #include "photos-collection-manager.h"
 #include "photos-dlna-renderers-manager.h"
 #include "photos-dropdown.h"
@@ -41,8 +40,8 @@
 #include "photos-mode-controller.h"
 #include "photos-overview-searchbar.h"
 #include "photos-remote-display-manager.h"
+#include "photos-search-context.h"
 #include "photos-selection-controller.h"
-#include "photos-source-manager.h"
 
 
 struct _PhotosMainToolbarPrivate
@@ -511,7 +510,7 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
   GMenu *preview_menu;
   GtkWidget *back_button;
   GtkWidget *menu_button;
-  GtkApplication *app;
+  GApplication *app;
   gboolean remote_display_available;
   GAction *remote_display_action;
 
@@ -532,11 +531,10 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
 
   /* Disable the remote-display-current action if the dLeyna services are not
    * available */
-  app = photos_application_new ();
+  app = g_application_get_default ();
   remote_display_action = g_action_map_lookup_action (G_ACTION_MAP (app), "remote-display-current");
   remote_display_available = photos_dlna_renderers_manager_is_available ();
   g_simple_action_set_enabled (G_SIMPLE_ACTION (remote_display_action), remote_display_available);
-  g_object_unref (app);
 
   g_signal_connect (priv->item_mngr, "active-changed", G_CALLBACK (photos_main_toolbar_item_active_changed), self);
 }
@@ -704,12 +702,16 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
 {
   PhotosMainToolbarPrivate *priv;
   GMenu *selection_menu;
-  GtkApplication *app;
+  GApplication *app;
   GtkBuilder *builder;
+  PhotosSearchContextState *state;
 
   self->priv = photos_main_toolbar_get_instance_private (self);
   priv = self->priv;
 
+  app = g_application_get_default ();
+  state = photos_search_context_get_state (PHOTOS_SEARCH_CONTEXT (app));
+
   gtk_orientable_set_orientation (GTK_ORIENTABLE (self), GTK_ORIENTATION_VERTICAL);
   gtk_widget_show (GTK_WIDGET (self));
 
@@ -717,10 +719,8 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
   gtk_container_add (GTK_CONTAINER (self), priv->toolbar);
   gtk_widget_show (priv->toolbar);
 
-  app = photos_application_new ();
   priv->gear_menu = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (app), "gear-menu"));
   priv->search = g_action_map_lookup_action (G_ACTION_MAP (app), "search");
-  g_object_unref (app);
 
   builder = gtk_builder_new ();
   gtk_builder_add_from_resource (builder, "/org/gnome/photos/selection-menu.ui", NULL);
@@ -734,10 +734,10 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
   photos_header_bar_set_selection_menu (PHOTOS_HEADER_BAR (priv->toolbar),
                                         GD_HEADER_BUTTON (priv->selection_menu));
 
-  priv->col_mngr = photos_collection_manager_dup_singleton ();
+  priv->col_mngr = g_object_ref (state->col_mngr);
   priv->item_mngr = photos_item_manager_dup_singleton ();
 
-  priv->src_mngr = photos_source_manager_dup_singleton ();
+  priv->src_mngr = g_object_ref (state->src_mngr);
   g_signal_connect_object (priv->src_mngr,
                            "active-changed",
                            G_CALLBACK (photos_main_toolbar_set_toolbar_title),
