@@ -34,6 +34,7 @@
 #include <tracker-sparql.h>
 #include <libgd/gd.h>
 
+#include "photos-query.h"
 #include "photos-tracker-queue.h"
 #include "photos-utils.h"
 
@@ -259,6 +260,88 @@ photos_utils_create_thumbnail (GFile *file, GCancellable *cancellable, GError **
 }
 
 
+static GIcon *
+photos_utils_get_thumbnail_icon (const gchar *uri)
+{
+  GError *error;
+  GFile *file = NULL;
+  GFile *thumb_file = NULL;
+  GFileInfo *info = NULL;
+  GIcon *icon = NULL;
+  const gchar *thumb_path;
+
+  file = g_file_new_for_uri (uri);
+
+  error = NULL;
+  info = g_file_query_info (file, G_FILE_ATTRIBUTE_THUMBNAIL_PATH, G_FILE_QUERY_INFO_NONE, NULL, &error);
+  if (error != NULL)
+    {
+      g_warning ("Unable to fetch thumbnail path for %s: %s", uri, error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  thumb_path = g_file_info_get_attribute_byte_string (info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH);
+  thumb_file = g_file_new_for_path (thumb_path);
+  icon = g_file_icon_new (thumb_file);
+
+ out:
+  g_clear_object (&thumb_file);
+  g_clear_object (&info);
+  g_clear_object (&file);
+  return icon;
+}
+
+
+GIcon *
+photos_utils_get_icon_from_cursor (TrackerSparqlCursor *cursor)
+{
+  GIcon *icon = NULL;
+  gboolean is_remote = FALSE;
+  const gchar *identifier;
+  const gchar *mime_type;
+  const gchar *rdf_type;
+
+  identifier = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_IDENTIFIER, NULL);
+  if (identifier != NULL)
+    {
+      if (g_str_has_prefix (identifier, "facebook:") || g_str_has_prefix (identifier, "flickr:"))
+        is_remote = TRUE;
+    }
+
+  if (!is_remote)
+    {
+      const gchar *uri;
+
+      uri = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_URI, NULL);
+      if (uri != NULL)
+        icon = photos_utils_get_thumbnail_icon (uri);
+    }
+
+  if (icon != NULL)
+    goto out;
+
+  mime_type = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_MIME_TYPE, NULL);
+  if (mime_type != NULL)
+    icon = g_content_type_get_icon (mime_type);
+
+  if (icon != NULL)
+    goto out;
+
+  rdf_type = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_RDF_TYPE, NULL);
+  if (mime_type != NULL)
+    icon = photos_utils_icon_from_rdf_type (rdf_type);
+
+  if (icon != NULL)
+    goto out;
+
+  icon = g_themed_icon_new ("image-x-generic");
+
+ out:
+  return icon;
+}
+
+
 const gchar *
 photos_utils_dot_dir (void)
 {
@@ -280,84 +363,6 @@ photos_utils_dot_dir (void)
 }
 
 
-GdkPixbuf *
-photos_utils_embed_image_in_frame (GdkPixbuf *source_image,
-                                   const gchar *frame_image_path,
-                                   GtkBorder *slice_width,
-                                   GtkBorder *border_width)
-{
-  GError *error = NULL;
-  GdkPixbuf *ret_val;
-  GtkCssProvider *provider;
-  GtkStyleContext *context;
-  GtkWidgetPath *path;
-  cairo_surface_t *surface;
-  cairo_t *cr;
-  gchar *css_str;
-  int dest_height;
-  int dest_width;
-  int source_height;
-  int source_width;
-
-  source_width = gdk_pixbuf_get_width (source_image);
-  source_height = gdk_pixbuf_get_height (source_image);
-
-  dest_width = source_width +  border_width->left + border_width->right;
-  dest_height = source_height + border_width->top + border_width->bottom;
-
-  css_str = g_strdup_printf (".embedded-image { border-image: url(\"%s\") %d %d %d %d / %dpx %dpx %dpx %dpx }",
-                             frame_image_path,
-                             slice_width->top, slice_width->right, slice_width->bottom, slice_width->left,
-                             border_width->top, border_width->right, border_width->bottom, border_width->left);
-  provider = gtk_css_provider_new ();
-  gtk_css_provider_load_from_data (provider, css_str, -1, &error);
-
-  if (error != NULL)
-    {
-      g_warning ("Unable to create the thumbnail frame image: %s", error->message);
-      g_error_free (error);
-      g_free (css_str);
-
-      return g_object_ref (source_image);
-    }
-
-  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, dest_width, dest_height);
-  cr = cairo_create (surface);
-
-  context = gtk_style_context_new ();
-  path = gtk_widget_path_new ();
-  gtk_widget_path_append_type (path, GTK_TYPE_ICON_VIEW);
-
-  gtk_style_context_set_path (context, path);
-  gtk_style_context_add_provider (context, GTK_STYLE_PROVIDER (provider), 600);
-
-  gtk_style_context_save (context);
-  gtk_style_context_add_class (context, "embedded-image");
-
-  gtk_render_frame (context, cr,
-                    0, 0,
-                    dest_width, dest_height);
-
-  gtk_style_context_restore (context);
-
-  gtk_render_icon (context, cr,
-                   source_image,
-                   border_width->left, border_width->top);
-
-  ret_val = gdk_pixbuf_get_from_surface (surface, 0, 0, dest_width, dest_height);
-
-  cairo_surface_destroy (surface);
-  cairo_destroy (cr);
-
-  gtk_widget_path_unref (path);
-  g_object_unref (provider);
-  g_object_unref (context);
-  g_free (css_str);
-
-  return ret_val;
-}
-
-
 GQuark
 photos_utils_error_quark (void)
 {
